package production

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/production/model"
)

type plan struct {
	Table
	issue       issue
	order       string `data:"1"`
	fields      string `data:"title,content,plan_status,issue_count,task_count,case_count,begin_time,end_time,process_time,finish_time,user_uid"`
	create      string `name:"create title* content* begin_time:select@date end_time:select@date" role:"leader"`
	modify      string `name:"modify title* content* begin_time*:select@date end_time*:select@date" role:"leader"`
	remove      string `name:"remove" role:"leader"`
	process     string `name:"process" role:"leader"`
	finish      string `name:"finish" role:"leader"`
	issueCreate string `name:"issueCreate title* content* issue_type:select level:select" role:"leader,worker"`
}

func (s plan) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, kit.ArgDef(arg, kit.Simple(model.BEGIN_TIME, m.Time(), model.END_TIME, m.Time("720h"))...)...)
	s.SendMessage(m, "", "")
}
func (s plan) List(m *ice.Message, arg ...string) {
	isLeader := s.IsLeader(m)
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		switch PlanStatus(kit.Int(value[model.PLAN_STATUS])) {
		case PlanCreate:
			if isLeader {
				button = append(button, s.Process, s.Modify, s.Remove)
			}
		case PlanProcess:
			button = append(button, s.IssueCreate)
			if isLeader {
				button = append(button, s.Finish)
			}
		case PlanFinish:
			if isLeader && m.Option(model.MARKET_UID) == "" {
				button = append(button, s.MarketInsert)
			}
			kit.If(len(arg) == 2, func() { s.DoneMessage(m) })
		}
		m.PushButton(button...)
	}).Display("")
	kit.If(!isLeader, func() { m.Action() })
}
func (s plan) Process(m *ice.Message, arg ...string) {
	s.changeStatus(m, PlanCreate, PlanProcess)
}
func (s plan) Finish(m *ice.Message, arg ...string) {
	count := m.Cmd(s.issue, s.Select, "plan_uid = ? AND status != ?", m.Option(model.UID), IssueFinish).Length()
	if m.WarnNotValid(count > 0, kit.Format("还有 %v 个未完成的需求", count)) {
		return
	}
	s.changeStatus(m, PlanProcess, PlanFinish)
}
func (s plan) IssueCreate(m *ice.Message, arg ...string) {
	m.Cmd(s.issue, s.Create, arg, model.PLAN_UID, m.Option(model.UID))
}

func init() { ice.TeamCtxCmd(plan{}) }

func (s plan) changeStatus(m *ice.Message, from, to PlanStatus) {
	s.ChangeStatus(m, int(from), int(to), m.ActionKey()+"_time", m.Time())
}

type PlanStatus int

const (
	PlanCreate PlanStatus = iota
	PlanProcess
	PlanFinish
)

var PlanStatusList = map[PlanStatus]string{
	PlanCreate:  "create",
	PlanProcess: "process",
	PlanFinish:  "finish",
}

func (s PlanStatus) String() string { return PlanStatusList[s] }
